# Migrate to version 1.0.0-rc+2 from 1.0.0-rc+1

## Change Graph context name and URIs generat by OpenSILEX

Update model graph from sets/<model_name> to set/<model_name>. Update URI generation by using "id" instead of "set".

OpenSILEX allow to easily perform some update/migration operation by just executing commands.

Running command line on OpenSILEX executable .jar file (User/Admin oriented)

This update performs SPARQL graph and MongoDB collection renaming, after major changes on URI generation and graph/collection renaming (commit 46a276118a6b8c47669997c7cb5ee4aca4524141 on master branch).

Before running this command you should perform a backup of you data.

```
# Connect to a console in OpenSILEX server
# From the directory which contains your OpenSILEX executable .jar file
java -jar opensilex.jar --CONFIG_FILE=<config_file> system run-update org.opensilex.migration.GraphAndCollectionMigration

```

**[IMPORTANT]**: after update success, run sparql reset-ontologies command in order to refresh ontologies.

```
java -jar opensilex.jar --CONFIG_FILE=<config_file> sparql reset-ontologies
```

## Organization

### Changes on the ontology

You are concerned about this change if you already have organizations and/or facilities.
In this version, some concepts of the ontology concerning the organizations have changed.
`vocabulary:Infrastructure` has become `foaf:Organization` and `vocabulary:InfrastructureFacility`
has become `vocabulary:Facility`. Two properties have also changed and need to be updated.

### SPARQL query to run

If you have organizations hosted by some facilities, and/or experiments associated with
at least one organization, and/or custom types for facilities, then you will need to run the following
SPARQL queries :

```sparql
delete {
    ?s ?p ?o
} insert {
    ?s <http://www.opensilex.org/vocabulary/oeso#isHosted> ?o
} where {
    ?s ?p ?o.
    values (?p) { (<http://www.opensilex.org/vocabulary/oeso#hasFacility>) }
}

delete {
    ?s ?p ?o
} insert {
    ?s <http://www.opensilex.org/vocabulary/oeso#usesOrganization> ?o
} where {
    ?s ?p ?o.
    values (?p) { (<http://www.opensilex.org/vocabulary/oeso#hasInfrastructure>) }
}

delete {
  ?uri ?p <http://www.opensilex.org/vocabulary/oeso#InfrastructureFacility>
} insert {
  ?uri ?p <http://www.opensilex.org/vocabulary/oeso#Facility>
} where {
  ?uri ?p <http://www.opensilex.org/vocabulary/oeso#InfrastructureFacility>
}

delete {
  ?uri ?p <http://www.opensilex.org/vocabulary/oeso#Infrastructure>
} insert {
  ?uri ?p <http://www.opensilex.org/vocabulary/oeso#Organization>
} where {
  ?uri ?p <http://www.opensilex.org/vocabulary/oeso#Infrastructure>
}
```

##### Update queries 12-10-2022

Modifications have been created in global graph not in specific instance organization graph <http://www.opensilex.org/<instance_prefix>/set/organization> (Ex: <http://www.opensilex.org/sunagri/set/organization>).

It may cause side effect suh as :

- Organisation are not associated to their facilities
- Facilities are not associated to their organisations

The following queries may fix this behaviour.

```
PREFIX vocabulary: <http://www.opensilex.org/vocabulary/oeso#>
 
DELETE{
  ?uri vocabulary:isHosted ?uri2
}
INSERT{
  GRAPH <http://www.opensilex.org/<instance_prefix>/set/organization>{
   ?uri vocabulary:isHosted ?uri2
  }
}
WHERE{
  ?uri vocabulary:isHosted ?uri2
}

```

PREFIX vocabulary: <http://www.opensilex.org/vocabulary/oeso#>

DELETE{
  ?uri vocabulary:usesOrganization ?uri2
}
INSERT{
  GRAPH <http://www.opensilex.org/<instance_prefix>/set/organization>{
   ?uri vocabulary:usesOrganization ?uri2
  }
}
WHERE{
  ?uri vocabulary:usesOrganization ?uri2
}

```

#### Specific ontology file changes

If you have a specific ontology file deployed to your instance, you will need to do
some checks :

- Subclasses of `vocabulary:Infrastructure` need to be marked as subclasses of `foaf:Organization` instead.
- Subclasses of `vocabulary:InfrastructureFacilities` need to be marked as subclasses of `vocabulary:Facilities` instead.
- If you have defined the following custom types, you need to remove them :
  - `vocabulary:Installation`
  - `vocabulary:ResearchUnit`
  - `vocabulary:ExperimentalUnit`
