# Migrate to version 1.0.0-rc+3 from 1.0.0-rc+2

- [Migrate to version 1.0.0-rc+3 from 1.0.0-rc+2](#migrate-to-version-100-rc3-from-100-rc2)
  - [Factors categories](#factors-categories)
    - [Change about API Factor Category Model](#change-about-api-factor-category-model)
      - [SPARQL script to run](#sparql-script-to-run)
  - [Change about Files system managment](#change-about-files-system-managment)
    - [Previous configuration example](#previous-configuration-example)
    - [New configuration example](#new-configuration-example)
    - [Document file system migration : change from local to gridfs](#document-file-system-migration--change-from-local-to-gridfs)

## Factors categories

### Change about API Factor Category Model

From version 1.0.0-rc+3, we will suggest to use [PECO Ontology](https://www.ontobee.org/ontology/PECO?iri=http://purl.obolibrary.org/obo/PECO_0001001) to represent experimental factor instead of AGROVOC thesaurus.
If you want to keep Agrovoc categories, add a ticket on [OpenSILEX Mantis](http://opensilex.org/mantisbt). 
Otherwise, run SPARQL script to update.

Also in this version, you will be able to add new or specialize existing factor categories.

#### SPARQL script to remove Agrovoc categories  

If you already have create factors in experiments, you are concerned by this upgrade instruction.

Remove unnecessary graph

```sparql
  CLEAR GRAPH <http://aims.fao.org/aos/agrovoc/factors>
```

Remove links to previous factor category

```sparql
PREFIX vocabulary: <http://www.opensilex.org/vocabulary/oeso#>

DELETE{
  ?factor  vocabulary:hasCategory  ?category
}WHERE{
  ?factor  vocabulary:hasCategory  ?category
}
```

You must fill new category factors in existing factors.

## Change about Files system management

File system configuration have changed. The previous one defined a storageBasePath that was used to provide a default local file system path.
We have improved filesystem connections, so you should use MongoDB storage for document file for easier maintenance. 
In order to do this, if you have existing documents, please follow [the migration steps](#for-mongo-document-file-system-migration--change-from-local-to-gridfs),
otherwise if you don't have any documents use [this config file](#final-configuration-file-example-for-mongo-management).

### Previous configuration example

```yml
# ------------------------------------------------------------------------------
# Configuration for module: FileStorageModule (FileStorageConfig)
file-system:
  # File storage access (FileStorageService)
  fs:
    # Base path for file storage (String)
    storageBasePath: ../../opensilex-data
```

### New configuration example

Now file system will be based on a set of defined connections. Connections can be used as default or for custom api path as `/documents` or `/datafiles`.
More information can be found in [File system configuration](../installation/configuration/file-system.md)

The new version of the previous configuration is this one. Three file system connections are availble for now : _LOCAL FILE SYSTEM, GRIDFS, IRODS_.
The new `defaultFS` is mandatory, if not set the temporary filesystem will be used. Empty default connection must not be used in production environment.

```yml
# ------------------------------------------------------------------------------
# Configuration for module: FileStorageModule (FileStorageConfig)
file-system:
  # File storage service (FileStorageService)
  fs:
    config:
      # Default file system storage (String)
      defaultFS: local
      # Map of file storage connection definition by identifier (Map<String,FileStorageConnection>)
      connections:
        local:
          # Service implementation class for: local (FileStorageConnection)
          implementation: org.opensilex.fs.local.LocalFileSystemConnection
          config:
            # Base path for file storage (String)
            basePath: ../../opensilex-data
```

### For Mongo Document file system migration : change from local to gridfs

OpenSILEX allow to easily perform some update/migration operation by just executing commands.

Running command line on OpenSILEX executable .jar file (User/Admin oriented)

Before running this command you should perform a backup of you documents.

Export your documents using the current file system configuration.

Example :

```yaml
# ------------------------------------------------------------------------------
# Configuration for module: FileStorageModule (FileStorageConfig)
file-system:
  # File storage service (FileStorageService)
  fs:
    config:
      # Default file system storage (String)
      defaultFS: local
      # Map of file storage connection definition by identifier (Map<String,FileStorageConnection>)
      connections:
        local:
          # Service implementation class for: local (FileStorageConnection)
          implementation: org.opensilex.fs.local.LocalFileSystemConnection
          config:
            # Base path for file storage (String)
            basePath: ../../opensilex-data
```

```bash
# Connect to a console in OpenSILEX server
# From the directory which contains your OpenSILEX executable .jar file
java -jar opensilex.jar --CONFIG_FILE=<config_file> system run-update org.opensilex.migration.ExportDocumentFilesFromLocalFSToGRIDFS
```

Import your documents using the current new file system configuration by adding gridfs configuration.

Example :

```yaml
# ------------------------------------------------------------------------------
# Configuration for module: FileStorageModule (FileStorageConfig)
file-system:
  # File storage service (FileStorageService)
  fs:
    config:
      # Default file system storage (String)
      defaultFS: gridfs
      # Map of file storage connection definition by identifier (Map<String,FileStorageConnection>)
      connections:
        local:
          # Service implementation class for: local (FileStorageConnection)
          implementation: org.opensilex.fs.local.LocalFileSystemConnection
          config:
            # Base path for file storage (String)
            basePath: ../../opensilex-data
        gridfs:
          implementation: org.opensilex.fs.gridfs.GridFSConnection
          config:
            host: localhost
            port: 27017
            database: opensilex
```

```
# Connect to a console in OpenSILEX server
# From the directory which contains your OpenSILEX executable .jar file
java -jar opensilex.jar --CONFIG_FILE=<config_file> system run-update org.opensilex.migration.ImportDocumentFilesFromLocalFSToGRIDFS
```

### Final configuration file example for Mongo management

```yaml
# ------------------------------------------------------------------------------
# Configuration for module: FileStorageModule (FileStorageConfig)
file-system:
  # File storage service (FileStorageService)
  fs:
    config:
      # Default file system storage (String)
      defaultFS: local
      # Map of file storage connection definition by identifier (Map<String,FileStorageConnection>)
      connections:
        local:
          # Service implementation class for: local (FileStorageConnection)
          implementation: org.opensilex.fs.local.LocalFileSystemConnection
          config:
            # Base path for file storage (String)
            basePath: ../../opensilex-data
        gridfs:
          implementation: org.opensilex.fs.gridfs.GridFSConnection
          config:
            host: localhost
            port: 27017
            database: opensilex
      # Map of custom path connection management (Map<String,String>)
      customPath:
        documents/: gridfs
```


## Experiments and species

The species of an experiment need no longer to be specified at the creation. They are automatically inferred from the
scientific objects attached to the experiment. 

For ended experiments you will need to run the following command:

```
java -jar opensilex.jar --CONFIG_FILE=<config_file> system run-update org.opensilex.migration.experiment.UpdateExperimentSpecies
```

This will update the species of all experiments of the instance.