# Migrate to 1.0.0

## Reset ontologies

New concepts and properties were introduced in this version. You will need to update the ontologies on your triple
store :

```
java -jar opensilex.jar --CONFIG_FILE=<config_file> sparql reset-ontologies
```

## Configuration file

### Shared resource instances

In the case Shared Resource Instances (SRI) are defined in your configuration file, you have to update the declarations. 

Previously, a shared resource instance was configured like this :

```yaml
core:
sharedResourceInstances:
  - uri: http://phenome.inrae.fr/resources/rest
    label:
        fr: "PHENOME"
        en: "PHENOME"
    accountName: guest@opensilex.org
    accountPassword: guest
```

The `apiUrl` key was added to distinguish the identifier of the SRI from the API endpoint. This caused problems when
changing the API endpoint of an SRI. For most cases, you should be able to duplicate the `uri` key, like this :

```yaml
core:
  sharedResourceInstances:
    - uri: http://phenome.inrae.fr/resources/rest
      apiUrl: http://phenome.inrae.fr/resources/rest # Duplicate the "uri" line and rename it "apiUrl"
      label:
        fr: "PHENOME"
        en: "PHENOME"
      accountName: guest@opensilex.org
      accountPassword: guest
```

## Migration of several links from Accounts to Persons
> **Description :**

Since the last release, users are split into accounts and Persons.
With this release, several data are now linked to persons, in order to be more flexible and have more contact data.
With these changes, we have to migrate data to ensure that it corresponds to the new model.

following predicates are concerned :
- projects : Oeso:hasAdministrativeContact, Oeso:hasScientificContact, Oeso:hasCoordinator
- experiment : Oeso:hasScientificSupervisor, Oeso:hasTechnicalSupervisor
- device : Oeso:personInCharge
- Provenance : operator (this is not a predicate since it's store in the NoSQl database, but it also has to be migrated)

> **Migration execution :**

```shell
# Connect to a console in OpenSILEX server
# From the directory which contains your OpenSILEX executable .jar file
java -jar opensilex.jar --CONFIG_FILE=<config_file> system run-update org.opensilex.migration.ObjectMigrationFromAccountToPerson
```

See the migration `org.opensilex.migration.ObjectMigrationFromAccountToPerson`
from [migration_command.md](../how-to/migration_command.md) for additional notes

## To activate the Recherche Data Gouv module

See [developer documentation for Recherche Data Gouv](../specs/dataverse.md)
